#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateRequestFile = exports.generate = exports.createControllers = exports.getActionName = exports.getControllerName = exports.generateService = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const handlebars_1 = require("handlebars");
const controller_1 = require("./controller");
const service_1 = require("./service");
const utils_1 = require("../utils");
const generate_model_file_1 = require("../generate-model-file");
const fs_1 = require("fs");
const generate_use_request_file_1 = require("../generate-use-request-file");
const rimraf_1 = __importDefault(require("rimraf"));
const configJson = utils_1.loadConfig();
// 扩展模版命令toUpperCase
handlebars_1.registerHelper("toUpperCase", function (str) {
    return str.toUpperCase();
});
// 扩展模版命令toLowerCase
handlebars_1.registerHelper("toLowerCase", function (str) {
    return str.toLowerCase();
});
handlebars_1.registerHelper("replace", function (context, findStr, replaceStr) {
    return context.replace(findStr, replaceStr);
});
/**
 * 生成服务
 */
function generateService(config) {
    if (config.serviceDir && fs_1.existsSync(config.serviceDir)) {
        rimraf_1.default.sync(config.serviceDir);
    }
    if (config.controllerDir && fs_1.existsSync(config.controllerDir)) {
        rimraf_1.default.sync(config.controllerDir);
    }
    if (config.modelDir && fs_1.existsSync(config.modelDir)) {
        rimraf_1.default.sync(config.modelDir);
    }
    const { gateway, services, swagger } = config;
    utils_1.info("-------------------------");
    utils_1.info("Gatewat地址", gateway);
    utils_1.info("Swagger地址", swagger);
    if (services && Object.keys(services).length) {
        utils_1.info("服务模式", "多服务");
        // 多服务模式
        return Object.entries(services).map(([key, service]) => ({
            key: key,
            name: service,
            url: `${gateway}/${service}/${swagger}`,
            gateway: config.name,
            config,
        }));
    }
    else {
        utils_1.info("服务模式", "单服务");
        // 单服务模式
        return [
            {
                key: "",
                name: "",
                gateway: config.name,
                url: `${gateway}/${swagger}`,
                config,
            },
        ];
    }
}
exports.generateService = generateService;
/**
 * 获取控制器名称
 */
function getControllerName(path, currentTag, tags) {
    try {
        const [controller] = path.match(/(?<=\b\api\/).*?(?=\/)/g) ||
            path.match(/(?<=\b\api\/).*/g) ||
            path.match(/(?<=\/).*?(?=\/)/g) ||
            path.match(/(?<=\/).*/g);
        return controller
            .split("/")
            .map((x) => x.replace(/^\S/, (s) => s.toUpperCase()))
            .join("");
    }
    catch (a) {
        throw new Error(`路径:${path}不符合规范`);
    }
}
exports.getControllerName = getControllerName;
/**
 * 获取行为器名称
 */
function getActionName(operation) {
    return operation.replace(/Using.*?$/, "");
}
exports.getActionName = getActionName;
function getAliasName(config, service, key) {
    if (!config.alias) {
        return;
    }
    if (Array.isArray(config.alias)) {
        const target = config.alias.find((x) => x.service === service && x.from === key);
        if (target) {
            return target.to;
        }
    }
    else {
        if (config.alias.from === key) {
            return config.alias.to;
        }
    }
}
function getPropertyTypeOpenAPI2(schema) {
    switch (true) {
        case !!schema.originalRef:
            if (schema.originalRef.startsWith("Map«"))
                return;
            return schema.originalRef
                .replace(/^Page«/, "")
                .replace(/^Iterable«/, "")
                .replace(/»$/, "[]");
        case schema.type === "array":
            const type = getPropertyTypeOpenAPI2(schema.items);
            return type && `${type}[]`;
    }
}
function getPropertyTypeOpenAPI3(schema) {
    switch (true) {
        case !!schema.$ref:
            const ref = schema.$ref.replace("#/components/schemas/", "");
            if (ref.startsWith("Map"))
                return;
            return ref.startsWith("Page")
                ? ref.replace(/^Page/, "").replace(/$/, "[]")
                : ref;
        case schema.type === "array":
            const type = getPropertyTypeOpenAPI3(schema.items);
            return type && `${type}[]`;
    }
}
function getActionReponseShema(service, responses) {
    if (!service.config.model) {
        return;
    }
    const response = responses["200"];
    if (response && response.schema) {
        return getPropertyTypeOpenAPI2(response.schema);
    }
    if (response &&
        response.content &&
        response.content["application/json"] &&
        response.content["application/json"]["schema"]) {
        return getPropertyTypeOpenAPI3(response.content["application/json"]["schema"]);
    }
}
/**
 * 获取Action列表
 * @param paths
 */
function createControllers(service, controllers, paths, tags) {
    Object.entries(paths)
        .filter(([key]) => key.startsWith("/api") || key.startsWith(`/${service.name}`))
        .map(([key, config]) => ({
        path: key.replace(new RegExp(`^\/${service.name}\/`), "/"),
        config,
    }))
        .forEach(({ path, config }) => {
        // 接口行为
        Object.entries(config).forEach(([method, { summary, description, tags: currentTag, operationId, responses },]) => {
            // 获取Controller Name
            const getController = service.config.controllerResolver
                ? service.config.controllerResolver
                : getControllerName;
            const controllerName = getController(path, currentTag, tags);
            const aliasName = getAliasName(service.config, service.key, controllerName);
            const controller = aliasName || controllerName;
            // 获取Action Name
            const getAction = service.config.actionResolver
                ? service.config.actionResolver
                : getActionName;
            const action = getAction(operationId, method, path);
            const filename = controller
                .replace(/([A-Z])/g, "-$1")
                .replace(/^-/g, "")
                .toLowerCase();
            // 查询并创建控制器
            let target = controllers.find((x) => x.controller === filename);
            // 控制器不存在则自动创建
            if (!target) {
                target = {
                    gateway: service.gateway,
                    controller: filename,
                    filename: filename,
                    controllerClass: `${controller}Controller`,
                    serviceClass: `${controller}Service`,
                    actions: [],
                };
                controllers.push(target);
            }
            // 添加控制器行为
            target.actions.push({
                path,
                controller,
                action: (action || method).replace(/-(\w)/g, ($, $1) => $1.toUpperCase()),
                schema: getActionReponseShema(service, responses),
                defaultAction: !action,
                method: method.replace(/^\S/, (s) => s.toUpperCase()),
                comment: summary !== null && summary !== void 0 ? summary : description,
            });
        });
    });
}
exports.createControllers = createControllers;
/**
 * 生成配置文件
 * @param service
 */
function generate(service) {
    return node_fetch_1.default(service.url, { method: "GET" })
        .then((res) => res.json()) // expecting a json response
        .then(({ tags, paths, definitions, components, }) => __awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        utils_1.info("-------------------------");
        utils_1.info("服务名称", service.name || "无");
        utils_1.info("服务路径", service.url);
        utils_1.info("-------------------------");
        // 控制器列表
        const controllers = [];
        // 填装控制器列表
        createControllers(service, controllers, paths, tags);
        // 生产文件
        controller_1.generateControllerFiles(service, controllers);
        service_1.generateServiceFiles(service, controllers);
        if (definitions && ((_a = service === null || service === void 0 ? void 0 : service.config) === null || _a === void 0 ? void 0 : _a.model)) {
            generate_model_file_1.generateOpenAPI2ModelFiles(service, definitions);
        }
        if (components && ((_b = service === null || service === void 0 ? void 0 : service.config) === null || _b === void 0 ? void 0 : _b.model)) {
            generate_model_file_1.generateOpenAPI3ModelFiles(service, components);
        }
    }));
}
exports.generate = generate;
function generateRequestFile() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!configJson) {
            throw new Error("无法找到配置文件");
        }
        const generateGatewayFile = (config) => generateService(config).map(generate);
        // 多网关处理
        const configList = Array.isArray(configJson) ? configJson : [configJson];
        yield Promise.all(configList.map(generateGatewayFile).reduce((r, i) => r.concat(i), []));
        configList.forEach((config) => {
            if (!!config.useRequestFile) {
                generate_use_request_file_1.generateUseRequestFile(config.useRequestFile, {
                    dir: config.serviceDir,
                    alias: config.serviceAlias,
                });
                utils_1.info("更新use-request文件完成");
            }
        });
    });
}
exports.generateRequestFile = generateRequestFile;
