"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateUseRequestFile = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
let generatedServices;
/**
 *
 * @param declaration
 * @param option
 */
function generateUseRequestFile(filePath, option) {
    if (!generatedServices) {
        generatedServices = generateServices(generateServicePaths(option));
    }
    if (generatedServices) {
        const file = generateFile(generatedServices);
        fs_1.default.writeFileSync(path_1.default.resolve(filePath), file, "utf-8");
    }
}
exports.generateUseRequestFile = generateUseRequestFile;
/**
 * 生成服务路径
 * @param option
 * @returns
 */
function generateServicePaths(option) {
    // 服务文件列表
    const servicePaths = [];
    // 遍历服务目录
    const walk = (dir) => {
        fs_1.default.readdirSync(dir).forEach(function (file) {
            // 修正windows路径符号问题
            const fullpath = path_1.default.join(dir, file).replace(/\\/g, "/");
            const stat = fs_1.default.statSync(fullpath);
            if (stat.isFile() && fullpath.endsWith(".service.ts")) {
                servicePaths.push(fullpath);
            }
            else if (stat.isDirectory()) {
                const subdir = path_1.default.join(dir, file);
                walk(subdir);
            }
        });
    };
    walk(path_1.default.resolve(option.dir));
    // 返回服务文件路径
    return servicePaths.map((servicePath) => servicePath.replace(path_1.default.resolve(option.dir).replace(/\\/g, "/"), option.alias));
}
/**
 * 生成服务项
 * @param paths
 * @returns
 */
function generateServices(paths) {
    const toCaseString = (str = "") => str
        .replace(/-(\w)/g, ($, $1) => $1.toUpperCase())
        .replace(/^\S/, (s) => s.toUpperCase());
    return paths.map((filePath) => {
        const [name] = /[^\\]+(?=\.service\.ts$)/.exec(toCaseString(path_1.default.basename(filePath))) ||
            [];
        const [group] = /(?<=^.\/http\/services\/)(.*?)(?=\/.*?\.service\.ts$)/.exec(filePath) ||
            [];
        return {
            name: `${toCaseString(name)}Service`,
            group: toCaseString(group),
            path: filePath,
        };
    });
}
/**
 * 生成导入代码
 * @param services
 * @param placeholder
 * @returns
 */
function generateImportCode(services, placeholder = "") {
    const getImportName = (service) => service.group
        ? `${service.name} as ${service.group}_${service.name}`
        : service.name;
    return services
        .map((service) => `import { ${getImportName(service)} } from '${service.path.replace(/\.ts$/g, "")}'`)
        .join(`\r\n${placeholder}`);
}
/**
 * 生成服务列表
 * @param services
 * @param placeholder
 * @returns
 */
function generateServiceCode(services, placeholder = "  ") {
    const generateServices = (items, placeholder) => {
        return `${Object.entries(items)
            .map(([key, name]) => `${key}:${name}`)
            .join(`,\r\n${placeholder}`)}`;
    };
    const generateGroups = () => {
        const groups = services.reduce((result, item) => {
            result[item.group] = result[item.group] || {};
            result[item.group][item.name] = `${item.group}_${item.name}`;
            return result;
        }, {});
        return `${Object.entries(groups)
            .map(([group, items]) => `${group}: {
    ${generateServices(items, " ".repeat(4))}
  }`)
            .join(",\r\n  ")} `;
    };
    if (services.some((service) => !!service.group)) {
        return `const serviceMap = {
  ${generateGroups()}
}`;
    }
    else {
        return `const serviceMap = {
  ${services.map((service) => service.name).join(`,\r\n${placeholder}`)}
}`;
    }
}
function generateFile(services) {
    const importCode = generateImportCode(services);
    const serviceCode = generateServiceCode(services);
    return `${importCode}

${serviceCode}
      
export function useRequest<T>(
  select: (services: typeof serviceMap) => { new (): T }
): T {
  const service = select(serviceMap)
  return new service()
}
  `.replace("\n", "");
}
